import 'dart:convert';

import 'package:cached_network_image/cached_network_image.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:zhiying_comm/util/turn_chain/turn_chain_util.dart';
import 'package:zhiying_base_widget/dialog/global_dialog/intellect_search_goods_dialog/model/has_goods_dialog_style_model.dart';
import 'package:zhiying_base_widget/dialog/global_dialog/intellect_search_goods_dialog/model/intellect_dialog_data_model.dart';
import 'package:zhiying_comm/zhiying_comm.dart';
import 'package:zhiying_comm/util/pdd_auth/pdd_auth.dart';

class IntellectSearchGoodsDialog extends StatefulWidget {
  final IntellectDialogDataModel model;

  const IntellectSearchGoodsDialog({Key key, this.model}) : super(key: key);

  @override
  _IntellectSearchGoodsDialogState createState() => _IntellectSearchGoodsDialogState();
}

class _IntellectSearchGoodsDialogState extends State<IntellectSearchGoodsDialog> {
  HasGoodsDialogStyleModel _styleModel;

  ///打开详情
  void openGoodsDetails() {
    Clipboard.setData(ClipboardData(text: ""));
    String provider = widget.model.provider;
    String goodId = widget.model.itemId;
    RouterUtil.route(SkipModel(skipIdentifier: "goods_details"), {"provider": provider, "good_id": goodId}, context).then((data) {
      Navigator.pop(context);
    });
  }

  /// 跳转购买
  void openUrl() async {
    ///跳转购买
    UserInfoModel userInfo = UserInfoNotifier.staitcUserInfo;

    /// 登录
    if (userInfo?.token == null || userInfo.token == '') {
      print('need login...');
      RouterUtil.goLogin(context);
      return;
    } else {
      print("平台" + widget?.model?.provider ?? "");
      // if(!widget?.model?.couponUrl.contains("http")){
      //   widget?.model?.couponUrl="https:"+widget?.model?.couponUrl;
      // }
      // widget?.model?.couponUrl=widget?.model?.couponUrl.replaceAll("\\", "");
      print("链接" + widget?.model?.couponUrl ?? "");
      Logger.log('智能弹窗有商品情况跳转数据 = ${widget?.model?.toJson()}');

      String tempProvider = widget?.model?.provider ?? 'taobao';

      /// 判断淘宝渠道是否授权
      if (tempProvider == GlobalConfig.PROVIDER_TB || tempProvider == GlobalConfig.PROVIDER_TM) {
        bool isAuth = await TaobaoAuth.auth(context);

        ///关闭弹窗
        if (Navigator.canPop(context)) {
          Navigator.pop(context);
        }
        if (!isAuth) return;
      } else if (tempProvider == GlobalConfig.PROVIDER_PDD) {
        /// 拼多多
        bool isNeedAuth = await PddAuth.auth(context, isShare: '0');

        ///关闭弹窗
        if (Navigator.canPop(context)) {
          Navigator.pop(context);
        }
        if (isNeedAuth) return;
      } else {
        ///关闭弹窗
        if (Navigator.canPop(context)) {
          Navigator.pop(context);
        }
      }

      /// 改成统一跳转
      await TurnChainUtil.jumpNative(
        context,
        provider: widget?.model?.provider ?? 'taobao',
        openAppUrl: widget?.model?.couponUrl,
        appUrl: widget?.model?.appUrl,
        webUrl: widget?.model?.couponUrl,
      );
      // if (widget?.model?.provider == GlobalConfig.PROVIDER_TB || widget?.model?.provider == GlobalConfig.PROVIDER_TM) {
      //   TradeResult result;
      //   if (Platform.isAndroid) {
      //     result = await FlutterAlibc.openByUrl(url: widget?.model?.couponUrl ?? "", backUrl: "alisdk://");
      //   } else if (Platform.isIOS) {
      //     result = await FlutterAlibc.openByUrl(url: widget?.model?.couponUrl ?? "");
      //   }
      //   Logger.debug('${result.errorCode}   ${result.errorMessage} ');
      //   print("链接" + widget?.model?.couponUrl ?? "");
      //   print("错误码" + result.errorCode);
      //   print("原因" + result.errorMessage);
      // } else if (widget?.model?.provider == GlobalConfig.PROVIDER_JD) {
      //   /// 京东
      //   // Jdsdk.openUrl(url: openAppUrl);
      //   Logger.log('走了京东打开');
      // } else {
      //   /// 其它
      //   Logger.log('走了其它打开方式');
      //   // await FlutterAlibc.openByUrl(url: widget?.model?.couponUrl, openType: AlibcOpenType.AlibcOpenTypeNative);
      // }
    }
  }

  @override
  void initState() {
    requestStyle();
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      backgroundColor: Colors.transparent,
      body: InkWell(
        onTap: () {
          Navigator.of(context).pop();
        },
        child: Column(
          mainAxisAlignment: MainAxisAlignment.center,
          children: <Widget>[
            GestureDetector(
              onTap: () {},
              child: Stack(
                children: <Widget>[
                  Container(
                    margin: EdgeInsets.only(top: 19, left: 48, right: 48),
                    padding: EdgeInsets.only(top: 38, left: 15, right: 15, bottom: 24),
                    decoration: BoxDecoration(color: HexColor.fromHex(_styleModel?.bgColor ?? "#FFFFFF"), borderRadius: BorderRadius.circular(16)),
                    width: double.infinity,
                    child: Column(
                      children: <Widget>[_buildTopItem(), _buildBottomItem()],
                    ),
                  ),
                  Container(
                    height: 38,
                    child: Center(
                        child: Container(
                      height: 38,
                      width: 108,
                      child: CachedNetworkImage(
                        imageUrl: _styleModel?.titleImg ?? "",
                        fit: BoxFit.fill,
                      ),
                    )),
                  ),
                ],
              ),
            ),
            SizedBox(
              height: 16,
            ),
            IconButton(
                iconSize: 36,
                icon: Icon(
                  Icons.cancel,
                  color: Colors.white,
                ),
                onPressed: () {
                  Navigator.pop(context);
                })
          ],
        ),
      ),
    );
  }

  _buildTopItem() {
    return Row(
      crossAxisAlignment: CrossAxisAlignment.start,
      children: <Widget>[
        Container(
          width: 80,
          height: 80,
          decoration:
              BoxDecoration(borderRadius: BorderRadius.circular(6), image: DecorationImage(image: CachedNetworkImageProvider(widget.model?.thumbnail ?? ""), fit: BoxFit.fill)),
        ),
        SizedBox(
          width: 9.5,
        ),
        Expanded(
            child: Container(
          child: Column(
            mainAxisSize: MainAxisSize.max,
            mainAxisAlignment: MainAxisAlignment.spaceBetween,
            children: <Widget>[
              Column(
                children: <Widget>[
                  Text(
                    widget.model?.title ?? "",
                    maxLines: 2,
                    overflow: TextOverflow.ellipsis,
                    style: TextStyle(color: Colors.black, fontSize: 12),
                  ),
                  SizedBox(
                    height: 5,
                  ),
                  Row(
                    children: <Widget>[
                      (widget.model?.couponPrice ?? "0") == "0"
                          ? Container()
                          : Container(
                              margin: EdgeInsets.only(right: 16),
                              decoration: BoxDecoration(color: Colors.red, borderRadius: BorderRadius.circular(2)),
                              padding: EdgeInsets.all(2),
                              child: Row(
                                mainAxisSize: MainAxisSize.min,
                                children: <Widget>[
                                  Container(
                                    width: 15,
                                    height: 15,
                                    decoration: BoxDecoration(color: HexColor.fromHex(_styleModel?.moduleList?.coupon?.bgColor ?? "")),
                                    child: Center(
                                      child: Text(
                                        _styleModel?.moduleList?.coupon?.content ?? "",
                                        style: TextStyle(color: HexColor.fromHex(_styleModel?.moduleList?.coupon?.fontColor ?? ""), fontSize: 10),
                                      ),
                                    ),
                                  ),
                                  Padding(
                                    padding: const EdgeInsets.only(left: 4, right: 4),
                                    child: Text(
                                      "¥" + (widget.model?.couponPrice ?? ""),
                                      style: TextStyle(color: HexColor.fromHex(_styleModel?.moduleList?.couponPrice?.fontColor), fontSize: 10),
                                    ),
                                  )
                                ],
                              ),
                            ),
                      Container(
                        decoration: BoxDecoration(color: HexColor.fromHex(_styleModel?.moduleList?.commission?.bgColor), borderRadius: BorderRadius.circular(2)),
                        padding: EdgeInsets.all(2),
                        child: Row(
                          mainAxisSize: MainAxisSize.min,
                          children: <Widget>[
                            SizedBox(
                              height: 15,
                            ),
                            Center(
                              child: Padding(
                                padding: const EdgeInsets.only(left: 4, right: 4),
                                child: Text(
                                  (_styleModel?.moduleList?.commission?.content ?? "") + " ¥" + (widget.model?.commission ?? ""),
                                  style: TextStyle(color: HexColor.fromHex(_styleModel?.moduleList?.commission?.fontColor ?? ""), fontSize: 10),
                                ),
                              ),
                            )
                          ],
                        ),
                      )
                    ],
                  ),
                ],
              ),
              SizedBox(
                height: 5,
              ),
              Row(
                crossAxisAlignment: CrossAxisAlignment.end,
                children: <Widget>[
                  Padding(
                    padding: const EdgeInsets.only(bottom: 2),
                    child: Text(
                      _styleModel?.moduleList?.couponAfter?.content ?? "",
                      style: TextStyle(fontSize: 10, color: HexColor.fromHex(_styleModel?.moduleList?.couponAfterPrice?.fontColor ?? "")),
                    ),
                  ),
                  Padding(
                    padding: const EdgeInsets.only(bottom: 2, left: 4, right: 4),
                    child: Text(
                      "¥",
                      style: TextStyle(fontSize: 10, color: HexColor.fromHex(_styleModel?.moduleList?.couponAfterPrice?.fontColor ?? "")),
                    ),
                  ),
                  Expanded(
                    child: Text(
                      // formatNum((num.tryParse(widget?.model?.price ?? "0") - num.tryParse(widget?.model?.couponPrice ?? "0")), 2),
                      widget?.model?.finalPrice ?? '0',
                      maxLines: 1,
                      style: TextStyle(
                        fontSize: 16,
                        color: HexColor.fromHex(_styleModel?.moduleList?.couponAfterPrice?.fontColor),
                      ),
                    ),
                  )
                ],
              )
            ],
          ),
        ))
      ],
    );
  }

  // 取两位小数，不进行四舍五入
  String formatNum(var num, int position) {
    String rlt = num?.toString() ?? '0';
    if (rlt.indexOf('.') == -1) return rlt;
    if ((num.toString().length - num.toString().lastIndexOf(".") - 1) < position) {
      //小数点后有几位小数
      rlt = num.toStringAsFixed(position).substring(0, num.toString().lastIndexOf(".") + position + 1).toString();
    } else {
      rlt = num.toString().substring(0, num.toString().lastIndexOf(".") + position + 1).toString();
    }
    return rlt;
  }

  _buildBottomItem() {
    return Column(
      children: <Widget>[
        SizedBox(
          height: 24,
        ),
        InkWell(
          child: Row(
            children: <Widget>[
              Expanded(
                  child: Container(
                margin: EdgeInsets.only(left: 10, right: 10),
                decoration: BoxDecoration(color: HexColor.fromHex(_styleModel?.moduleList?.saveBtn?.bgColor ?? ""), borderRadius: BorderRadius.circular(50)),
                child: Center(
                    child: Padding(
                  padding: const EdgeInsets.all(12.0),
                  child: Text( // 立省按钮文字
                    (_styleModel?.moduleList?.saveBtn?.content ?? "") +
                        " ¥" + widget?.model?.earnPrice ?? '0' ,
                        // formatNum((num.tryParse(widget?.model?.couponPrice ?? "0") ?? 0) + num.tryParse(widget?.model?.commission ?? '0'), 2),
                    style: TextStyle(color: HexColor.fromHex(_styleModel?.moduleList?.saveBtn?.fontColor), fontSize: 12),
                  ),
                )),
              ))
            ],
          ),
          onTap: () {
            openUrl();
          },
        ),
        SizedBox(
          height: 16,
        ),
        InkWell(
          child: Row(
            children: <Widget>[
              Expanded(
                  child: Container(
                margin: EdgeInsets.only(left: 10, right: 10),
                decoration: BoxDecoration(color: HexColor.fromHex(_styleModel?.moduleList?.viewGoodsBtn?.bgColor ?? ""), borderRadius: BorderRadius.circular(50)),
                child: Center(
                    child: Padding(
                  padding: const EdgeInsets.all(12.0),
                  child: Text(
                    _styleModel?.moduleList?.viewGoodsBtn?.content ?? "",
                    style: TextStyle(color: HexColor.fromHex(_styleModel?.moduleList?.viewGoodsBtn?.fontColor ?? ""), fontSize: 12),
                  ),
                )),
              ))
            ],
          ),
          onTap: () {
            openGoodsDetails();
          },
        )
      ],
    );
  }

  void requestStyle() {
    NetUtil.request("/api/v1/cfg/clipboard_identified", onSuccess: (data) {
      print(data);
      _styleModel = HasGoodsDialogStyleModel.fromJson(json.decode(data['clipboard_identified']));
      setState(() {});
    });
  }

  @override
  void dispose() {
    Clipboard.setData(ClipboardData(text: ""));
    super.dispose();
  }
}
