import 'dart:io';

import 'package:cached_network_image/cached_network_image.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:permission_handler/permission_handler.dart';
import 'package:zhiying_base_widget/dialog/global_dialog/notification_setting_dialog/notification_setting_model.dart';
import 'package:zhiying_base_widget/dialog/tip_dialog/tip_dialog.dart';
import 'package:zhiying_base_widget/widgets/home/home_quick_entry/cached_network_image_util.dart';
import 'package:zhiying_comm/zhiying_comm.dart';
import 'dart:convert' as convert;
import 'package:package_info/package_info.dart';

class NotificationSettingDialog extends StatelessWidget {
  final NotificationSettingModel model;

  const NotificationSettingDialog(this.model, {Key key}) : super(key: key);

  static Future show(BuildContext context) async {
    Map result = await NetUtil.post('/api/v1/mod/pub.flutter.push_dialog', method: NetMethod.GET);
    if (!EmptyUtil.isEmpty(result)) {
      var modListData = result['data']['mod_list'][0]['data'];
      if (!EmptyUtil.isEmpty(modListData)) {
        // return EmptyPageModel.fromJson(jsonDecode(modListData));
        //
        // String d = json['data']['data'];
        Map<String, dynamic> da = Map<String, dynamic>.from(convert.jsonDecode(modListData));
        NotificationSettingModel model = NotificationSettingModel.fromJson(Map<String, dynamic>.from(da));

        await showCupertinoDialog(context: context, builder: (_) => NotificationSettingDialog(model));

        if (!await Permission.notification.isGranted) {
          await Permission.notification.request();
          if (await Permission.notification.isDenied) {
            var packageInfo = await PackageInfo.fromPlatform();
            var result = await showDialog(
                context: context,
                child: TipDialog(
                  btnText: "去打开",
                  content: "系统检查到您还未开启通知权限，可通过以下指引打开通知权限\n 设置 -> 应用管理 -> " + (packageInfo?.appName ?? "") + " -> 通知管理",
                ));
            if (result != null && result) {
              if (Platform.isAndroid) {
                NativeUtil.openAppSettings();
              } else {
                openAppSettings();
              }
            }
          }
        }

        if (!await Permission.storage.isGranted) {
          await Permission.storage.request();
        }
      }
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      backgroundColor: Colors.transparent,
      body: Container(
        width: double.infinity,
        child: Column(
          crossAxisAlignment: CrossAxisAlignment.center,
          mainAxisAlignment: MainAxisAlignment.center,
          children: <Widget>[
            Container(
              width: 280,
              // height: 382,
              decoration: BoxDecoration(
                color: Colors.white,
                borderRadius: BorderRadius.circular(10),
              ),
              child: Column(
                children: <Widget>[
                  Stack(
                    children: <Widget>[
                      ClipRRect(
                        borderRadius: BorderRadius.only(topLeft: Radius.circular(10), topRight: Radius.circular(10)),
                        child: CachedNetworkImage(
                          fit: BoxFit.fitWidth,
                          imageUrl: model?.bgStackImg ?? '',
                        ),
                      ),
                      Transform.translate(
                        offset: Offset(0, -15),
                        child: CachedNetworkImage(
                          imageUrl: model?.bgOnStackImg ?? '',
                        ),
                      ),
                    ],
                  ),
                  Padding(
                    padding: const EdgeInsets.all(8.0),
                    child: Text(
                      model?.title ?? '',
                      style: TextStyle(
                        fontSize: 16,
                        fontWeight: FontWeight.bold,
                        color: HexColor.fromHex(model?.titleColor ?? '#FF4242'),
                      ),
                    ),
                  ),
                  Padding(
                    padding: const EdgeInsets.only(left: 8.0, right: 8.0, bottom: 4),
                    child: Text(
                      model?.subtitle ?? '',
                      style: TextStyle(
                        fontSize: 13,
                        fontWeight: FontWeight.normal,
                        color: HexColor.fromHex(model?.subtitleColor ?? '#FF4242'),
                      ),
                    ),
                  ),
                  Column(
                    children: model?.list.map((e) => _createItem(e)).toList() ?? [],
                  ),
                  GestureDetector(
                    onTap: () async {
                      Navigator.pop(context, true);
                      return true;
                    },
                    child: Container(
                      margin: EdgeInsets.only(top: 10, bottom: 10),
                      width: 120,
                      height: 60,
                      child: CachedNetworkImage(
                        imageUrl: model?.btnImg ?? '',
                      ),
                    ),
                  )
                ],
              ),
            ),
            GestureDetector(
              child: Container(
                width: 30,
                height: 30,
                margin: EdgeInsets.only(top: 10),
                child: CachedNetworkImage(
                  imageUrl: model?.deleteImg ?? '',
                ),
              ),
              onTap: () {
                Navigator.pop(context);
              },
            )
          ],
        ),
      ),
    );
  }

  Widget _createItem(NotificationSettingItemModel itemModel) {
    return Padding(
      padding: EdgeInsets.only(left: 30, right: 30, top: 4, bottom: 4),
      child: Row(
        mainAxisAlignment: MainAxisAlignment.start,
        crossAxisAlignment: CrossAxisAlignment.start,
        children: <Widget>[
          Container(
              margin: EdgeInsets.only(right: 6, top: 2),
              width: 20,
              height: 20,
              child: CachedNetworkImage(
                imageUrl: itemModel?.icon ?? '',
              )),
          Expanded(
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              children: <Widget>[
                Text(
                  itemModel?.title ?? '',
                  style: TextStyle(
                    fontSize: 14,
                    fontWeight: FontWeight.bold,
                    color: HexColor.fromHex(model?.listItemTitleColor ?? '#333333'),
                  ),
                ),
                Text(
                  model?.subtitle ?? '',
                  style: TextStyle(
                    fontSize: 11,
                    fontWeight: FontWeight.normal,
                    color: HexColor.fromHex(model?.listItemSubtitleColor ?? '#999999'),
                  ),
                ),
              ],
            ),
          )
        ],
      ),
    );
  }
}

class NotificationSettingDialogNew extends StatelessWidget {
  List<PermissModel> permissModels;

  NotificationSettingDialogNew(this.permissModels, {Key key}) : super(key: key);

  static PermissModel storagePermissModel = PermissModel(
      permissType: PermissType.storage,
      title: "存储权限",
      subTitle: "用于分享照片、保存图片到相册",
      iconUrl: "https://alipic.lanhuapp.com/SketchPng7acf1ec3d1c3c98ba9b58d52a5d49ff3404a0b3f131100695dcd479c8dfa215b");

  static PermissModel notificationPermissModel = PermissModel(
      permissType: PermissType.notification,
      title: "通知权限",
      subTitle: "用于及时告知优惠动态",
      iconUrl: "https://alipic.lanhuapp.com/SketchPnga11a94a0c38949016e0fced74d683c08eaa1f58366cba616a40e51a9ebbe7146");

  static PermissModel locationPermissModel = PermissModel(
      permissType: PermissType.location,
      title: "定位权限",
      subTitle: "用于定位当前位置推荐附的优惠动态",
      iconUrl: "https://alipic.lanhuapp.com/SketchPng409f144b2042e81840e20c2d528b3f50994869e3f22b5d7d2b8a47abcdb72953");

  static PermissModel photosPermissModel = PermissModel(
      permissType: PermissType.photos,
      title: "图库访问权限",
      subTitle: "用于设置用户头像",
      iconUrl: "https://alipic.lanhuapp.com/SketchPngb4dfa520067c3387340a16e558ba8138e6b1466e7e3ce019b70ae7bcbc6dfc63");
  static PermissModel cameraPermissModel = PermissModel(
      permissType: PermissType.camera,
      title: "摄像头访问权限",
      subTitle: "用于拍照设置用户头像",
      iconUrl: "https://alipic.lanhuapp.com/SketchPng404acd457590d35856e730a1d1c2a192a5ab570350130ea3d9b4fc84e51cc29d");

  static Future show(BuildContext context, {List<PermissModel> permissModels}) async {
    List<PermissModel> newPermissModels;
    if (permissModels == null) {
      newPermissModels = List();
      if (!await Permission.storage.isGranted) {
        newPermissModels.add(storagePermissModel);
      }
    } else {
      newPermissModels = permissModels;
    }
    if (newPermissModels.length == 0) {
      return;
    }
    await showCupertinoDialog(context: context, builder: (_) => NotificationSettingDialogNew(newPermissModels));
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      backgroundColor: Colors.transparent,
      body: Container(
        width: double.infinity,
        child: Column(
          crossAxisAlignment: CrossAxisAlignment.center,
          mainAxisAlignment: MainAxisAlignment.center,
          children: <Widget>[
            Container(
              width: 265,
              padding: EdgeInsets.all(20),
              // height: 382,
              decoration: BoxDecoration(
                color: Colors.white,
                borderRadius: BorderRadius.circular(10),
              ),
              child: Column(
                children: <Widget>[
                  Text(
                    "申请获取权限",
                    style: TextStyle(
                      fontSize: 15,
                      fontWeight: FontWeight.bold,
                      color: HexColor.fromHex('#FF4242'),
                    ),
                  ),
                  SizedBox(
                    height: 10,
                  ),
                  Text(
                    "		\t\t\t\t\t为了保证您可以正常使用APP, 更好地为您提供服务, 我们需要向您申请获得如下权限",
                    style: TextStyle(fontSize: 13, color: HexColor.fromHex("#FF666666")),
                  ),
                  SizedBox(
                    height: 10,
                  ),
                  Column(
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: permissModels.map((e) => _createItem(e)).toList() ?? [],
                  ),
                  SizedBox(
                    height: 20,
                  ),
                  GestureDetector(
                    onTap: () async {
                      for (var item in permissModels) {
                        if (item.permissType == PermissType.notification) {
                          if (!await Permission.notification.isGranted) {
                            await Permission.notification.request();
                            if (await Permission.notification.isDenied) {
                              if (Platform.isAndroid) {
                                await NativeUtil.openAppSettings();
                              } else {
                                openAppSettings();
                              }
                            }
                          }
                        } else if (item.permissType == PermissType.storage) {
                          if (!await Permission.storage.isGranted) {
                            await Permission.storage.request();
                          }
                          if (await Permission.storage.isPermanentlyDenied) {
                            openAppSettings();
                          }
                        } else if (item?.permissType == PermissType.location) {
                          await Permission.location.request();
                        }else if(item?.permissType==PermissType.camera){
                          await Permission.camera.request();
                        }else if(item?.permissType==PermissType.photos){
                          await Permission.photos.request();
                        }
                      }
                      Navigator.pop(context, true);
                    },
                    child: Container(
                        margin: EdgeInsets.only(top: 10, bottom: 10),
                        height: 38,
                        decoration: BoxDecoration(
                            borderRadius: BorderRadius.circular(8),
                            gradient:
                                LinearGradient(begin: Alignment.topCenter, end: Alignment.bottomCenter, colors: [HexColor.fromHex("#FFFF5632"), HexColor.fromHex("#FFFF4242")])),
                        child: Center(
                            child: Text(
                          "确认授权",
                          style: TextStyle(color: Colors.white, fontSize: 13),
                        ))),
                  )
                ],
              ),
            ),
            GestureDetector(
              onTap: () {
                Navigator.pop(context, false);
              },
              child: Container(alignment: Alignment.center, margin: EdgeInsets.only(top: 10), child: CloseButton(color: HexColor.fromHex("#FFFFFF"))),
            )
          ],
        ),
      ),
    );
  }

  Widget _createItem(PermissModel itemModel) {
    return Container(
      decoration: BoxDecoration(color: HexColor.fromHex("#FFFAFAFA"), borderRadius: BorderRadius.circular(8)),
      margin: EdgeInsets.only(top: 8),
      child: Padding(
        padding: EdgeInsets.only(left: 8, right: 8, top: 4, bottom: 4),
        child: Row(
          mainAxisAlignment: MainAxisAlignment.start,
          crossAxisAlignment: CrossAxisAlignment.center,
          children: <Widget>[
            Container(
                margin: EdgeInsets.only(right: 6, top: 2),
                width: 27,
                height: 27,
                child: CachedNetworkImage(
                  imageUrl: itemModel?.iconUrl ?? '',
                )),
            Expanded(
              child: Column(
                crossAxisAlignment: CrossAxisAlignment.start,
                children: <Widget>[
                  Text(
                    itemModel?.title ?? '',
                    style: TextStyle(
                      fontSize: 14,
                      fontWeight: FontWeight.bold,
                      color: HexColor.fromHex('#333333'),
                    ),
                  ),
                  Text(
                    itemModel?.subTitle ?? '',
                    style: TextStyle(
                      fontSize: 11,
                      fontWeight: FontWeight.normal,
                      color: HexColor.fromHex('#999999'),
                    ),
                  ),
                ],
              ),
            )
          ],
        ),
      ),
    );
  }
}

class PermissModel {
  String iconUrl;
  String title;
  String subTitle;
  PermissType permissType;

  PermissModel({this.iconUrl, this.title, this.subTitle, this.permissType});
}

enum PermissType { notification, storage, location,camera,photos}
