import 'dart:async';

import 'package:bloc/bloc.dart';
import 'package:zhiying_base_widget/pages/custom_page/bloc/bottom_nav_repository.dart';
import 'package:zhiying_base_widget/pages/custom_page/bloc/custom_item_page_repository.dart';
import 'custom_item_page_state.dart';
import 'custom_item_page_event.dart';
import 'package:zhiying_comm/zhiying_comm.dart';

class CustomItemPageBloc extends Bloc<CustomItemPageEvent, CustomItemPageState> {
  // CustomItemPageBloc() : super(CustomItemPageInitial());

  CustomItemPageRepository repository;

  CustomItemPageBloc(this.repository) : super(CustomItemPageInitial());
  BottomNavRepository _bottomNavRepository = new BottomNavRepository();



  @override
  Stream<CustomItemPageState> mapEventToState(
    CustomItemPageEvent event,
  ) async* {
    /// 初始化
    if (event is CustomItemPageInitEvent) {
      yield* _mapInitEventToState(event);
    }

    /// 下拉刷新
    if (event is CustomItemPageRefreshEvent) {
      yield* _mapRefreshEventToState(event);
    }

    /// 上拉更多
    if (event is CustomItemPageLoadEvent) {
      yield* _mapLoadEventToState(event);
    }

  }

  /// 初始化
  Stream<CustomItemPageState> _mapInitEventToState(CustomItemPageInitEvent event) async* {
    var cacheStyle = await repository.fetchCacheStyle();
    if (!EmptyUtil.isEmpty(cacheStyle)) {
      yield CustomItemPageLoadedState(model: cacheStyle);
    }
    var netStyle = await repository.fetchNetStyle();
    if (!EmptyUtil.isEmpty(netStyle) && netStyle is List) {
      yield CustomItemPageLoadedState(model: netStyle);
    } else {
      yield CustomItemPageInitErrorState();
    }
    // 检查校验是否需要重启App(底部模版问题)
    bool needRestart = await _bottomNavRepository.validateBottomNavUpdate();
    Logger.log('init 是否重新启动App = ${needRestart}');
    if (needRestart) {
      yield CustomItemPageAppRestartState();
    }
  }

  /// 下拉刷新
  Stream<CustomItemPageState> _mapRefreshEventToState(CustomItemPageRefreshEvent event) async* {
    var netStyle = await repository.fetchNetStyle();
    if (!EmptyUtil.isEmpty(netStyle)) {
      yield CustomItemPageLoadedState(model: netStyle);
      yield CustomItemPageRefreshSuccessState();
    } else {
      yield CustomItemPageRefreshErrorState();
      yield CustomItemPageErrorState();
    }
    // 检查校验是否需要重启App(底部模版问题)
    bool needRestart = await _bottomNavRepository.validateBottomNavUpdate();
    Logger.log('refresh 是否重新启动App = ${needRestart}');
    if (needRestart) {
      // App重启，不需要进行子Widget刷新
      yield CustomItemPageAppRestartState();
    } else {
      // 子Widget刷新，写在后面是为了等渲染完成
      yield CustomItemPageItemWidgetRefreshState();
    }
  }

  /// 上拉更多
  Stream<CustomItemPageState> _mapLoadEventToState(CustomItemPageLoadEvent event) async* {}
}
