import 'dart:async';
import 'dart:io';

import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:zhiying_base_widget/dialog/notification_dialog/notification_dialog.dart';
import 'package:zhiying_base_widget/pages/message_settings_page/model/message_settings_data_model.dart';
import 'package:zhiying_base_widget/pages/message_settings_page/model/message_settings_style_model.dart';
import 'package:zhiying_base_widget/utils/contants.dart';
import 'package:zhiying_base_widget/utils/mob_push_util.dart';

import 'package:zhiying_comm/zhiying_comm.dart';
import 'bloc/message_settings_bloc.dart';
import 'bloc/message_settings_event.dart';
import 'bloc/message_settings_repository.dart';
import 'bloc/message_settings_state.dart';

///
/// 消息设置
///
class MessageSettingsPage extends StatelessWidget {
  final Map<String, dynamic> data;

  MessageSettingsPage(this.data);

  @override
  Widget build(BuildContext context) {
    return BlocProvider<MessageSettingsBloc>(
      create: (_) => MessageSettingsBloc(MessageSettingsRepository())..add(MessageSettingsInitEvent(data)),
      child: _MessageSettingsPageContainer(),
    );
  }
}

class _MessageSettingsPageContainer extends StatefulWidget {
  @override
  __MessageSettingsPageContainerState createState() => __MessageSettingsPageContainerState();
}

class __MessageSettingsPageContainerState extends State<_MessageSettingsPageContainer> {
  bool isOpen = false;

  /// 列表点击
  void _onClickListItem(Settings model, MessageSettingsDataModel dataModel) {
    /// 跳转查看隐私协议
    if (model.type == 'hid_url') {
      RouterUtil.openWebview(dataModel.data[model.dataKeys[0]], context);
    }
  }

  /// 隐私开关
  void _switchOnChange(bool value) async {
    if (value) {
      await SharedPreferencesUtil.setStringValue(Constants.notificationAgree, "1");
      if (!await Permission.notification.isGranted) {
        bool isGranted = await Permission.notification.isGranted;
        if (!isGranted) {
          var result = await showDialog(
              context: context,
              child: NotificationDialog(
                isGranted: isGranted,
              ));
          if (result != null && result) {
            SharedPreferencesUtil.setStringValue(Constants.notificationAgree, "1");
            if (!await Permission.notification.isGranted) {
              if (Platform.isIOS) {
                await openAppSettings().then((value) {
                  getState();
                });
              } else {
                await NativeUtil.openAppSettings();
              }
            }
          }
        }
      }
      setAlias();
    } else {
      await SharedPreferencesUtil.setStringValue(Constants.notificationAgree, "0");
      MobPushUtil.deleteAlias();
    }
    getState();
    //BlocProvider.of<MessageSettingsBloc>(context).add(MessageSettingsUpdateEvent(value: value, key: key));
  }

  ///设置定向推送
  void setAlias() async {
    ///如果没有开启通知则设置别名
    UserInfoModel userInfo = UserInfoNotifier?.staitcUserInfo;
    var setting = await NativeUtil.getSetting();
    String masterId = setting['master_id'];
    Logger.log("我的Alias: " + masterId + "_" + userInfo?.userId);
    if (!EmptyUtil.isEmpty(userInfo.userId) && !EmptyUtil.isEmpty(masterId)) {
      print("设置标签" + masterId + "_" + userInfo.userId);
      await MobPushUtil.restartPush();
      await MobPushUtil.setAlias(masterId + "_" + userInfo.userId);
      SharedPreferencesUtil.setStringValue(Constants.isSetTag, "1");
    }
  }

  Timer timer;

  @override
  void initState() {
    getState();
    timer = Timer.periodic(Duration(seconds: 1), (timer) {
      getState();
    });
    super.initState();
  }

  @override
  void dispose() {
    timer?.cancel();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return BlocConsumer<MessageSettingsBloc, MessageSettingsState>(
      listener: (context, state) {},
      buildWhen: (prev, current) {
        if (current is MessageSettingsUpdateErrorState) {
          return false;
        }
        return true;
      },
      builder: (context, state) {
        if (state is MessageSettingsLoadedState) {
          return _buildMainWidget(state?.styleModel, state.dataModel);
        }
        return _buildEmptyWidget();
      },
    );
  }

  /// 空数据
  Widget _buildEmptyWidget() {
    return Scaffold(
      backgroundColor: HexColor.fromHex('#F9F9F9'),
      appBar: _buildAppBarWidget(null),
      body: Container(),
    );
  }

  /// 有数据
  Widget _buildMainWidget(MessageSettingsStyleModel model, MessageSettingsDataModel dataModel) {
    return Scaffold(
      backgroundColor: HexColor.fromHex(model?.bgColor ?? '#F9F9F9'),
      appBar: _buildAppBarWidget(model),
      body: Column(
        children: <Widget>[
          /// 隐私订单
          Container(
            margin: const EdgeInsets.only(top: 5),
            padding: const EdgeInsets.symmetric(horizontal: 12.5),
            color: HexColor.fromHex(model?.settingsBgColor ?? '#FFFFFF'),
            child: _buildHeadWidge(model?.settings, dataModel),
          ),

          /// 隐私协议
          Container(
            margin: const EdgeInsets.only(top: 5),
            padding: const EdgeInsets.symmetric(horizontal: 12.5),
            color: HexColor.fromHex(model?.settingsBgColor ?? '#FFFFFF'),
            child: _buildListWidget(model?.otherSettings, dataModel),
          )
        ],
      ),
    );
  }

  /// 根据列表生成list集合
  Widget _buildHeadWidge(List<Settings> model, MessageSettingsDataModel dataModel) {
    List<Widget> lists = [];

    int length = model?.length ?? 0;
    if (length == 0) {
      lists.add(Container());
    } else {
      model.forEach((element) {
        lists.add(GestureDetector(
          onTap: () => _onClickListItem(element, dataModel),
          behavior: HitTestBehavior.opaque,
          child: Container(
            margin: const EdgeInsets.symmetric(vertical: 15),
            child: Row(
              mainAxisAlignment: MainAxisAlignment.spaceBetween,
              crossAxisAlignment: CrossAxisAlignment.center,
              children: <Widget>[
                Column(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  mainAxisAlignment: MainAxisAlignment.center,
                  children: <Widget>[
                    Text(
                      element?.name ?? '',
                      style: TextStyle(color: HexColor.fromHex(element?.nameColor ?? '#333333'), fontSize: 13, fontWeight: FontWeight.bold),
                    ),
                    Visibility(
                      visible: !EmptyUtil.isEmpty(element?.subName),
                      child: Text(
                        element?.subName ?? '',
                        style: TextStyle(color: HexColor.fromHex(element?.subNameColor ?? '#999999'), fontSize: 11),
                      ),
                    )
                  ],
                ),
                Container(
                  width: 60,
                  height: 15,
                  child: Switch(
                    value: isOpen,
                    activeColor: HexColor.fromHex('#FF4B4B'),
                    // trackColor: HexColor.fromHex('#F9F9F9') ,
                    onChanged: (bool value) {
                      _switchOnChange(value);
                    },
                  ),
                ),
              ],
            ),
          ),
        ));
      });
    }

    return Column(
      children: lists,
    );
  }

  /// 根据列表根据列表生成List集合
  Widget _buildListWidget(List<Settings> model, MessageSettingsDataModel dataModel) {
    List<Widget> lists = [];

    int length = model?.length ?? 0;
    if (length == 0) {
      lists.add(Container());
    } else {
      model.forEach((element) {
        lists.add(GestureDetector(
          onTap: () => _onClickListItem(element, dataModel),
          behavior: HitTestBehavior.opaque,
          child: Container(
            margin: const EdgeInsets.symmetric(vertical: 15),
            child: Row(
              mainAxisAlignment: MainAxisAlignment.spaceBetween,
              children: <Widget>[
                Text(
                  element?.name ?? '',
                  style: TextStyle(color: HexColor.fromHex(element?.nameColor ?? '#333333'), fontSize: 13, fontWeight: FontWeight.bold),
                ),
                Row(
                  children: <Widget>[
                    Text(
                      element?.descText ?? '',
                      style: TextStyle(color: HexColor.fromHex(element?.descColor ?? '#999999'), fontSize: 13),
                    ),
                    const SizedBox(width: 7.5),
                    Icon(
                      Icons.arrow_forward_ios,
                      color: HexColor.fromHex(element?.descColor ?? '#999999'),
                      size: 14,
                    ),
                  ],
                )
              ],
            ),
          ),
        ));
      });
    }

    return Column(
      children: lists,
    );
  }

  /// appBar
  Widget _buildAppBarWidget(MessageSettingsStyleModel model) {
    return AppBar(
      backgroundColor: HexColor.fromHex(model?.appBarBgColor ?? '#FFFFFF'),
      brightness: Brightness.light,
      leading: IconButton(
        icon: Icon(
          Icons.arrow_back_ios,
          size: 22,
          color: HexColor.fromHex('#333333'),
        ),
        onPressed: () => Navigator.maybePop(context),
      ),
      title: Text(
        model?.appBarName ?? '隐私设置',
        style: TextStyle(color: HexColor.fromHex(model?.appBarNameColor ?? '#333333'), fontSize: 15, fontWeight: FontWeight.bold),
      ),
      centerTitle: true,
      elevation: 0,
    );
  }

  void getState() async {
    var result = await SharedPreferencesUtil.getStringValue(Constants.notificationAgree, defaultVal: "0");
    var per = await Permission.notification.isGranted;
    isOpen = (result == "1" && per) ? true : false;
    setState(() {});
  }
}
