import 'dart:async';
import 'dart:convert';

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:zhiying_base_widget/dialog/loading/loading.dart';
import 'package:zhiying_base_widget/pages/order_found_page/order_found_result/order_found_result_model.dart';
import 'package:zhiying_base_widget/pages/order_found_page/order_found_result/order_found_result_page.dart';
import 'package:zhiying_base_widget/pages/order_found_page/order_found_search_page/model/order_found_search_style.dart';
import 'package:zhiying_comm/util/base_bloc.dart';
import 'package:zhiying_comm/util/empty_util.dart';
import 'package:zhiying_comm/util/log/let_log.dart';
import 'package:zhiying_comm/util/net_util.dart';
import 'package:zhiying_comm/zhiying_comm.dart';

class OrderFoundSearchBloc extends BlocBase {
  StreamController streamController = StreamController.broadcast();

  Stream get outData => streamController.stream;

  OrderFoundSearchStyle style;

  loadStyle() {
    NetUtil.request("/api/v1/order/find/css", onSuccess: (data) {
      style = OrderFoundSearchStyle.fromJson(json.decode(data['data']));
      streamController.add(style);
      Logger.log(json.encode(data['data']));
    });
  }

  searchOrderById(BuildContext context, String id) {
    var params = {"oid": id};
    Loading.show(context);
    NetUtil.request("/api/v1/order/find/search", params: params, method: NetMethod.POST, onSuccess: (data) {
      Loading.dismiss();
      if (EmptyUtil.isEmpty(data)) {
        Navigator.push(
            context,
            MaterialPageRoute(
                builder: (_) => OrderFoundResultPage(
                      emptyOrderTitle: style?.notFindRuleCss?.title ?? '',
                      emptyOrderContent: style?.notFindRuleCss?.content ?? "",
                    )));
      } else {
        Navigator.push(
            context,
            MaterialPageRoute(
                builder: (_) => OrderFoundResultPage(
                      orderFoundResultModel: OrderFoundResultModel.fromJson(data),
                      emptyOrderTitle: style?.notFindRuleCss?.title ?? '',
                      emptyOrderContent: style?.notFindRuleCss?.content ?? "",
                    ))).then((value) {
          if (value != null && value == "success") {
            Navigator.pop(context);
          }
        });
      }
      streamController.add(style);
      Logger.log(json.encode(data['data']));
    }, onError: (e) {
      Loading.dismiss();
    });
  }

  @override
  void dispose() {
    // TODO: implement dispose
  }
}
