import 'package:flutter/material.dart';
import 'package:zhiying_base_widget/pages/order_found_page/order_found_search_page/order_found_search_bloc.dart';
import 'package:zhiying_comm/zhiying_comm.dart';

import 'model/order_found_search_style.dart';

///订单搜索页
class OrderFoundSearchPage extends StatefulWidget {
  @override
  _OrderFoundSearchPageState createState() => _OrderFoundSearchPageState();
}

class _OrderFoundSearchPageState extends State<OrderFoundSearchPage> {
  OrderFoundSearchBloc _bloc;
  OrderFoundSearchStyle style;
  TextEditingController textEditingController;

  @override
  void initState() {
    textEditingController = TextEditingController();
    _bloc = OrderFoundSearchBloc();
    _bloc.loadStyle();
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return StreamBuilder(
        stream: _bloc.outData,
        builder: (context, asyn) {
          if (asyn.data == null) {
            return Container(
              color: Colors.white,
            );
          }
          style = asyn.data;
          return Stack(
            children: [
              Container(
                color: HexColor.fromHex("#f9f9f9"),
              ),
              Container(
                height: 200,
                width: MediaQuery.of(context).size.width,
                decoration: BoxDecoration(
                    borderRadius: BorderRadius.only(bottomLeft: Radius.circular(6), bottomRight: Radius.circular(6)),
                    gradient: LinearGradient(
                        begin: Alignment.topCenter, end: Alignment.bottomCenter, colors: [HexColor.fromHex(style?.top?.bgColor), HexColor.fromHex(style?.top?.bgSubColor ?? '')])),
              ),
              Scaffold(
                backgroundColor: Colors.transparent,
                appBar: _buildAppBar(),
                body: Container(
                  child: ListView(
                    children: [_buildSearch(), _buildCenter(), _buildBottom()],
                  ),
                ),
              ),
            ],
          );
        });
  }

  _buildAppBar() {
    return AppBar(
      backgroundColor: Colors.transparent,
      elevation: 0,
      leading: Navigator.canPop(context)
          ? IconButton(
              icon: Icon(
                Icons.arrow_back_ios_rounded,
              ),
              iconSize: 18,
              onPressed: () {
                Navigator.pop(context);
              })
          : Container(),
      title: Text(
        style?.top?.str ?? '',
        style: TextStyle(fontSize: 17, color: HexColor.fromHex(style?.top?.strColor)),
      ),
      centerTitle: true,
      actions: [
        IconButton(
            icon: Icon(
              Icons.refresh,
              color: HexColor.fromHex(style?.top?.strColor),
            ),
            onPressed: () {
              _bloc?.loadStyle();
            })
      ],
    );
  }

  ///构建搜索
  _buildSearch() {
    return Container(
      height: 50,
      decoration: BoxDecoration(color: HexColor.fromHex(style?.searchCss?.bgColor ?? ''), borderRadius: BorderRadius.circular(10), border: Border.all(color: Colors.white)),
      margin: EdgeInsets.only(left: 10, right: 10, top: 10, bottom: 10),
      padding: EdgeInsets.only(left: 10, right: 4),
      child: Row(
        children: [
          Expanded(
              child: TextField(
            controller: textEditingController,
            decoration: InputDecoration(
                border: InputBorder.none,
                hintText: style?.searchCss?.str ?? '',
                hintStyle: TextStyle(color: HexColor.fromHex(style?.searchCss?.strColor ?? ""), fontSize: 14),
                isDense: true),
          )),
          GestureDetector(
            onTap: () {
              if (textEditingController?.text.trim().length == 0) {
                Fluttertoast.showToast(msg: "请输入搜索的订单编号");
                return;
              }
              FocusScope.of(context).requestFocus(new FocusNode());
              _bloc.searchOrderById(context, textEditingController.text);
            },
            child: Container(
              height: 42,
              alignment: Alignment.center,
              padding: EdgeInsets.only(left: 24, right: 24),
              decoration: BoxDecoration(
                  borderRadius: BorderRadius.circular(10),
                  gradient: LinearGradient(
                      begin: Alignment.centerLeft,
                      end: Alignment.centerRight,
                      colors: [HexColor.fromHex(style?.searchCss?.btnBgColor), HexColor.fromHex(style?.searchCss?.btnBgSubColor ?? '')])),
              child: Text(
                style?.searchCss?.btnStr ?? "",
                style: TextStyle(color: HexColor.fromHex(style?.searchCss?.btnStrColor ?? "")),
              ),
            ),
          ),
        ],
      ),
    );
  }

  _buildCenter() {
    return Container(
      margin: EdgeInsets.only(left: 10, right: 10, top: 10, bottom: 10),
      padding: EdgeInsets.only(left: 10, right: 10, top: 10, bottom: 10),
      decoration: BoxDecoration(color: HexColor.fromHex("#FFFFFF"), borderRadius: BorderRadius.circular(10)),
      child: Column(
        children: [
          Row(
            children: [
              Container(
                width: 4,
                height: 14,
                decoration: BoxDecoration(color: HexColor.fromHex("#FF2020"), borderRadius: BorderRadius.circular(10)),
                margin: EdgeInsets.only(right: 10),
              ),
              Text(
                style?.ruleCss?.str ?? "",
                style: TextStyle(color: HexColor.fromHex("#333333"), fontSize: 15, fontWeight: FontWeight.w500),
              )
            ],
          ),
          SizedBox(
            height: 16,
          ),
          GestureDetector(
            onTap: () {
              PhotoPreview.showPhotoPreviewByimages(context, [style?.ruleCss?.img ?? ""]);
            },
            child: CachedNetworkImage(
              imageUrl: style?.ruleCss?.img ?? '',
              fit: BoxFit.fitWidth,
            ),
          )
        ],
      ),
    );
  }

  _buildBottom() {
    return Container(
      margin: EdgeInsets.only(left: 10, right: 10, top: 0, bottom: 10),
      padding: EdgeInsets.only(left: 10, right: 10, top: 10, bottom: 10),
      decoration: BoxDecoration(color: HexColor.fromHex("#FFFFFF"), borderRadius: BorderRadius.circular(10)),
      child: Column(
        children: [
          Row(
            children: [
              Container(
                width: 4,
                height: 14,
                margin: EdgeInsets.only(right: 10),
                decoration: BoxDecoration(color: HexColor.fromHex("#FF2020"), borderRadius: BorderRadius.circular(10)),
              ),
              Text(
                style?.findRuleCss?.title ?? '',
                style: TextStyle(color: HexColor.fromHex("#333333"), fontSize: 15, fontWeight: FontWeight.w500),
              )
            ],
          ),
          SizedBox(
            height: 16,
          ),
          ListView(
            shrinkWrap: true,
            physics: NeverScrollableScrollPhysics(),
            children: [
              Text(
                style?.findRuleCss?.content ?? "",
                style: TextStyle(color: HexColor.fromHex("#999999"), fontSize: 12, fontWeight: FontWeight.w500),
              )
            ],
          )
        ],
      ),
    );
  }

}
