import 'dart:convert';

import 'package:flutter/material.dart';
import 'package:zhiying_base_widget/widgets/custom/ordedr_class/order_class_nav/order_class_style.dart';
import 'package:zhiying_comm/util/extension/color.dart';
import 'package:zhiying_comm/zhiying_comm.dart';

///订单分类
class OrderClassNav extends StatelessWidget {
  final Map<String, dynamic> model;

  OrderClassNav(this.model, {Key key}) : super(key: key);

  OrderClassStyle style;
  BuildContext context;

  @override
  Widget build(BuildContext context) {
    // Logger.log(model['data']);
    style = OrderClassStyle.fromJson(json.decode(model['data']));
    if (style == null) {
      return Container();
    }
    this.context = context;
    return Container(
      margin: EdgeInsets.only(
          left: double.tryParse(style?.leftRightMargin ?? "0"), right: double.tryParse(style?.leftRightMargin ?? "0"), top: double.tryParse(style?.topMargin ?? "0"), bottom: 0),
      padding: EdgeInsets.only(left: 10, right: 10, top: 10, bottom: 10),
      decoration: BoxDecoration(
          color: HexColor.fromHex(style?.bgColor ?? ''),
          borderRadius: BorderRadius.only(
              topLeft: Radius.circular(double.tryParse(style?.topLeftRadius ?? "0")),
              topRight: Radius.circular(double.tryParse(style?.topRightRadius ?? "0")),
              bottomLeft: Radius.circular(double.tryParse(style?.bottomLeftRadius ?? "0")),
              bottomRight: Radius.circular(double.tryParse(style?.bottomRightRadius ?? "0")))),
      child: Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          Row(
            children: [
              Container(
                width: 4,
                height: 14,
                decoration: BoxDecoration(color: HexColor.fromHex("#FF2020"), borderRadius: BorderRadius.circular(10)),
                margin: EdgeInsets.only(right: 10),
              ),
              Text(
                style?.title ?? "",
                style: TextStyle(color: HexColor.fromHex("#333333"), fontSize: 15, fontWeight: FontWeight.w500),
              )
            ],
          ),
          Padding(
            padding: const EdgeInsets.only(left: 15),
            child: Text(
              style?.subTitle ?? "",
              style: TextStyle(color: HexColor.fromHex("#999999"), fontSize: 12, fontWeight: FontWeight.w500),
            ),
          ),
          SizedBox(
            height: 10,
          ),
          createNav()
        ],
      ),
    );
  }

  createNav() {
    List<Widget> listWidget = List();
    int row = (style?.listStyle?.length / style?.columSize).ceil();
    for (var index = 0; index < row; index++) {
      listWidget.add(Row(
        children: _buildRow(index),
      ));
    }

    return Column(children: listWidget);
  }

  _buildRow(int index2) {
    List<Widget> listWidget = List();

    int currentIndex = index2 * style?.columSize;

    for (var index = 0; index < style?.columSize; index++) {
      var nowIndex = currentIndex + index;
      listWidget.add(Expanded(
        child: _buildItem(nowIndex),
      ));
    }

    return listWidget;
  }

  _buildItem(int currentIndex) {
    print(currentIndex.toString());
    if (currentIndex >= style?.listStyle?.length) {
      return Container();
    }

    var item = style.listStyle[currentIndex];
    print(currentIndex.toString() + item?.title);
    return GestureDetector(
      onTap: () {
        RouterUtil.route(SkipModel.fromJson(item.toJson() ?? ''), item.toJson(), context);
      },
      child: Container(
        margin: EdgeInsets.only(top: 10),
        child: Column(
          children: [
            CachedNetworkImage(
              imageUrl: item?.img ?? "",
              width: 40,
              height: 40,
            ),
            SizedBox(
              height: 6,
            ),
            Text(
              item?.title,
              style: TextStyle(color: HexColor.fromHex("#333333"), fontSize: 12),
            ),
            Text(
              item?.subTitle,
              style: TextStyle(color: HexColor.fromHex("#333333"), fontSize: 10),
            ),
          ],
        ),
      ),
    );
  }
}
