import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:zhiying_base_widget/widgets/base_state/base_state.dart';
import 'package:zhiying_base_widget/widgets/custom/multi_nav/cached_network_image_util.dart';
import 'package:zhiying_base_widget/widgets/custom/share_content/model/custom_share_content_model.dart';
import 'package:zhiying_base_widget/widgets/custom/share_content/share_content_sk.dart';
import 'package:zhiying_base_widget/widgets/share/models/share_alert_model.dart';
import 'package:zhiying_base_widget/widgets/share/models/share_data_model.dart';
import 'package:zhiying_base_widget/widgets/share/share_alert.dart';
import 'package:zhiying_comm/zhiying_comm.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'bloc/custom_share_content_bloc.dart';
import 'bloc/custom_share_content_repository.dart';
import 'package:flutter/cupertino.dart';

///
///  分享内容组件
///
class CustomShareContentWidget extends StatelessWidget {
  final Map<String, dynamic> model;

  const CustomShareContentWidget(this.model, {Key key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return BlocProvider(
      create: (_) => CustomShareContentBloc(repository: CustomShareContentRepository())..add(CustomShareContentInitEvent(model: model)),
      child: _CustomShareContentContainerWidget(),
    );
  }
}

class _CustomShareContentContainerWidget extends StatefulWidget {
  @override
  _CustomShareContentContainerWidgetState createState() => _CustomShareContentContainerWidgetState();
}

class _CustomShareContentContainerWidgetState extends BaseWidgetState<_CustomShareContentContainerWidget> {
  /// 分享方法
  void _clickShareButton(val) {
    showCupertinoModalPopup(
      context: context,
      builder: (context) => ShareAlert(ShareDataModel(content: val), 'pub.flutter.share_icon', isContentShow: false),
    );
  }

  /// 复制方法
  void _clickCopyButton(String val) {
    if (!EmptyUtil.isEmpty(val)) {
      Clipboard.setData(ClipboardData(text: val));
      Fluttertoast.showToast(msg: '复制成功');
    }
  }

  @override
  Widget build(BuildContext context) {
    return BlocConsumer<CustomShareContentBloc, CustomShareContentState>(
      listener: (prev, next) {},
      buildWhen: (context, state) {
        return true;
      },
      builder: (context, state) {
        Logger.log('商品详情刷新了 builder state = ' + state?.toString());

        /// 数据加载完毕
        if (state is CustomShareContentLoadedState) {
          return _getMainWidget(state?.model);
        }

        /// 异常
        if (state is CustomShareContentErrorState) {
          return _errorWidget();
        }

        /// 骨架图
        return _getSkeletonWidget();
      },
    );
  }

  /// 主视图
  Widget _getMainWidget(CustomShareContentModel model) {
    return Container(
        width: double.infinity,
        // height: 98,
        margin: EdgeInsets.only(
            top: ParseUtil.stringParseDouble(model?.topMargin),
            left: ParseUtil.stringParseDouble(model?.leftRightMargin, defVal: 12.5),
            right: ParseUtil.stringParseDouble(model?.leftRightMargin, defVal: 12.5)),
        decoration: BoxDecoration(
            // color: Colors.red,
            borderRadius: BorderRadius.only(
          bottomLeft: Radius.circular(ParseUtil.stringParseDouble(model?.bottomLeftRadius)),
          bottomRight: Radius.circular(ParseUtil.stringParseDouble(model?.bottomRightRadius)),
          topLeft: Radius.circular(ParseUtil.stringParseDouble(model?.topLeftRadius)),
          topRight: Radius.circular(ParseUtil.stringParseDouble(model?.topRightRadius)),
        )),
        child: Container(
          width: double.infinity,
          padding: const EdgeInsets.only(left: 12, right: 12, bottom: 14),
          decoration: BoxDecoration(
              color: HexColor.fromHex(model?.cardBgColor),
              border: Border.all(color: HexColor.fromHex(model?.cardBorderColor), width: 1),
              borderRadius: BorderRadius.all(Radius.circular(8))),
          child: Column(
            children: [
              // 标题
              Row(
                mainAxisAlignment: MainAxisAlignment.spaceBetween,
                crossAxisAlignment: CrossAxisAlignment.center,
                children: [
                  CachedNetworkImage(imageUrl: model?.scaleImageUrl, width: 36),
                  CachedNetworkImage(imageUrl: model?.titleImageUrl, width: 90),
                  SizedBox(
                    width: 36,
                  )
                ],
              ),
              SizedBox(height: 20),
              // 文案
              Text(
                model?.text ?? '',
                style: TextStyle(color: HexColor.fromHex(model?.textColor), fontSize: 15),
                maxLines: 4,
                overflow: TextOverflow.ellipsis,
              ),
              SizedBox(height: 22),
              // 复制按钮 or 分享 按钮
              Visibility(
                  visible: (model?.isShowShareFriend ?? '0') == '1',
                  // 仅仅复制按钮
                  replacement:
                      GestureDetector(onTap: () => _clickCopyButton(model?.text), child: CachedNetworkImage(imageUrl: model?.copyLongImageUrl ?? '', width: double.infinity)),
                  child: Row(
                    children: [
                      // 复制按钮
                      Flexible(
                          flex: 1,
                          child: GestureDetector(
                              onTap: () => _clickCopyButton(model?.text),
                              child: CachedNetworkImage(
                                imageUrl: model?.copyImageUrl,
                                width: double.infinity,
                              ))),
                      SizedBox(width: 20),
                      // 分享按钮
                      Flexible(
                          flex: 1,
                          child: GestureDetector(
                              onTap: () => _clickShareButton(model?.text),
                              child: CachedNetworkImage(
                                imageUrl: model?.shareImageUrl,
                                width: double.infinity,
                              )))
                    ],
                  ))
            ],
          ),
        ));
  }

  /// 骨架图
  Widget _getSkeletonWidget() {
    return CustomShareContentSkeleton();
  }

  /// 无数据视图 or 异常
  Widget _errorWidget() {
    return Container();
  }

  @override
  void refreshWidget(String value) {
    setState(() {});
  }
}
